{{
8X8 LED Matrix Driver
Copyright(C) 2008 Thomas Watson
Use it whenever, wherever.
}}
CON

  _clkmode        = xtal1 + pll16x           ' Feedback and PLL multiplier
  _xinfreq        = 5_000_000
fontType = 1                                 'the type of font. 1=AiGeneric Font 2=AiGeneric Font without first 32 chars
offset = 32                                  'character offset for font type 2

dat
messageData byte "Propeller Rules!", 255   'the message to display

obj
fonts : "Font_TECH"                        'font

var
byte msgIdx
byte lock
byte row
long fontTable
long refreshStack[100]
word frameBuffer[8]
word refreshBuffer[8]

pub main
fontTable := fonts.GetPtrToFontTable       'get font addr
dira[16]~                                  
lock := locknew                            'check out a lock for the framebuffers
if ina[16] == 1                            'for serial config, not yet implemented
  reboot
cognew(refresh, @refreshStack)             'start the cog that refreshes the display
waitcnt(clkfreq/10+cnt)
logic                                      'begin the main loop

pub refresh | time, i, temp
row := %11111110                           'set the row
dira[0..15]~~                              'set direction
outa[0..15]~                               'clear outputs
repeat 
  row := %11111110                         'set row
  repeat until not lockset(lock)           'lock the framebuffer
  repeat i from 0 to 7                     'display loop
    outa[8..15] := frameBuffer[i] >> 8     'set the data pins
    outa[0..7] := row                      'set the row pins
    row := row <- 1                        'shift row left(it doesnt seem to rotate like it should)
    row |= 1                               'add a 1 because of shifting
    waitcnt(clkfreq/500+cnt)               'wait for leds to light up
    outa[0..15]~                           'clear pins
  lockclr(lock)                            'unlock the framebuffer

pub logic | temp, address, i
repeat
  temp := getMsgChar                       'get a character of the message
  if temp == -1                            'if end of message, try again
    temp := getMsgChar 
  address := getFontAddr(temp)             'get the address of the character in the font table
  loadBuffer(address)                      'load the framebuffer with the character
  repeat 9                                 'loop to shift character into view
    repeat i from 0 to 7                   'shift the framebuffer 1 space to the left
      refreshBuffer[i] <<= 1              
    repeat until not lockset(lock)         'lock the framebuffer  
    wordmove(@frameBuffer, @refreshBuffer, 8) 'copy the framebuffer to the refresh cogs framebuffer
    lockclr(lock)                          'unlock the framebuffer
    waitcnt(clkfreq/6+cnt)                 'wait before shifting again

pub getMsgChar | charAddr
charAddr := @messageData+msgIdx            'get character address from the message string
if byte[charAddr] == 255                   'check if end of message
  msgIdx := 0                              'reset message index
  return -1                                'return -1 to signify end of message
msgIdx++                                   'increment the message index
return byte[charAddr]                      'return message character

pub getFontAddr(character)
if fontType == 1                           'check for font type
  return ((character)*8)+fontTable         'do some math to figure out the address of the character
elseif fontType == 2
  return ((character-offset)*8)+fontTable  

pub loadBuffer(fontAddress) | i
repeat i from 0 to 7                       
  'refreshBuffer[i] &= $FF00
  refreshBuffer[i] |= byte[fontAddress+(i)] 'copy the character to the framebuffer
